@@ -50,23 +50,26 @@
 #include "src/common/cpu_frequency.h"
 #include "src/common/forward.h"
 #include "src/common/macros.h"
+#include "src/common/node_features.h"
+#include "src/common/port_mgr.h"
 #include "src/common/run_command.h"
 #include "src/common/setproctitle.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_pack.h"
 #include "src/common/slurm_rlimits_info.h"
-#include "src/common/macros.h"
-#include "src/common/port_mgr.h"
 #include "src/common/spank.h"
 #include "src/common/stepd_api.h"
+#include "src/common/stepd_proxy.h"
 #include "src/common/xmalloc.h"
-#include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
+#include "src/conmgr/conmgr.h"
+
 #include "src/interfaces/acct_gather_energy.h"
 #include "src/interfaces/acct_gather_profile.h"
 #include "src/interfaces/auth.h"
 #include "src/interfaces/cgroup.h"
+#include "src/interfaces/conn.h"
 #include "src/interfaces/gpu.h"
 #include "src/interfaces/gres.h"
 #include "src/interfaces/hash.h"
@@ -105,18 +108,9 @@ static void _step_cleanup(stepd_step_rec_t *step, slurm_msg_t *msg, int rc);
 static void _process_cmdline(int argc, char **argv);
 
 static pthread_mutex_t cleanup_mutex = PTHREAD_MUTEX_INITIALIZER;
-static bool cleanup = false;
-
-/*
- *  List of signals to block in this process
- */
-int slurmstepd_blocked_signals[] = {
-	SIGINT,  SIGTERM, SIGTSTP,
-	SIGQUIT, SIGPIPE, SIGUSR1,
-	SIGUSR2, SIGALRM, SIGHUP, 0
-};
 
 /* global variable */
+uint32_t slurm_daemon = IS_SLURMSTEPD;
 slurmd_conf_t * conf;
 extern char  ** environ;
 
@@ -145,6 +139,7 @@ static int _foreach_ret_data_info(void *x, void *arg)
 
 static void *_rpc_thread(void *data)
 {
+	bool srun_agent = false;
 	agent_arg_t *agent_arg_ptr = data;
 	slurm_msg_t msg;
 	slurm_msg_t_init(&msg);
@@ -153,12 +148,25 @@ static void *_rpc_thread(void *data)
 	msg.flags = agent_arg_ptr->msg_flags;
 	msg.msg_type = agent_arg_ptr->msg_type;
 	msg.protocol_version = agent_arg_ptr->protocol_version;
+	msg.tls_cert = xstrdup(agent_arg_ptr->tls_cert);
 
 	slurm_msg_set_r_uid(&msg, agent_arg_ptr->r_uid);
 
+	srun_agent = ((msg.msg_type == SRUN_PING) ||
+		      (msg.msg_type == SRUN_JOB_COMPLETE) ||
+		      (msg.msg_type == SRUN_STEP_MISSING) ||
+		      (msg.msg_type == SRUN_STEP_SIGNAL) ||
+		      (msg.msg_type == SRUN_TIMEOUT) ||
+		      (msg.msg_type == SRUN_USER_MSG) ||
+		      (msg.msg_type == RESPONSE_RESOURCE_ALLOCATION) ||
+		      (msg.msg_type == SRUN_NODE_FAIL));
+
 	if (agent_arg_ptr->addr) {
 		msg.address = *agent_arg_ptr->addr;
-		if (slurm_send_only_node_msg(&msg)) {
+
+		if (msg.msg_type == SRUN_JOB_COMPLETE) {
+			slurm_send_msg_maybe(&msg);
+		} else if (slurm_send_only_node_msg(&msg) && !srun_agent) {
 			error("failed to send message type %d/%s",
 			      msg.msg_type, rpc_num2string(msg.msg_type));
 		}
@@ -219,23 +227,8 @@ static int _foreach_job_node_array(void *x, void *arg)
 	node_record_t *job_node_ptr = x;
 	int *table_index = arg;
 
-	config_record_t *config_ptr;
-	config_ptr = create_config_record();
-	config_ptr->boards = job_node_ptr->boards;
-	config_ptr->core_spec_cnt = job_node_ptr->core_spec_cnt;
-	config_ptr->cores = job_node_ptr->cores;
-	config_ptr->cpu_spec_list = xstrdup(job_node_ptr->cpu_spec_list);
-	config_ptr->cpus = job_node_ptr->cpus;
-	config_ptr->feature = xstrdup(job_node_ptr->features);
-	config_ptr->gres = xstrdup(job_node_ptr->gres);
-	config_ptr->node_bitmap = bit_alloc(node_record_count);
-	config_ptr->nodes = xstrdup(job_node_ptr->name);
-	config_ptr->real_memory = job_node_ptr->real_memory;
-	config_ptr->res_cores_per_gpu = job_node_ptr->res_cores_per_gpu;
-	config_ptr->threads = job_node_ptr->threads;
-	config_ptr->tmp_disk = job_node_ptr->tmp_disk;
-	config_ptr->tot_sockets = job_node_ptr->tot_sockets;
-	config_ptr->weight = job_node_ptr->weight;
+	config_record_t *config_ptr =
+		config_record_from_node_record(job_node_ptr);
 
 	*table_index = bit_ffs_from_bit(job_step_ptr->node_bitmap, *table_index);
 
@@ -281,35 +274,122 @@ static void _init_stepd_stepmgr(void)
 	reserve_port_stepmgr_init(job_step_ptr);
 
 	_setup_stepmgr_nodes();
+	node_features_build_active_list(job_step_ptr);
 
-	if (!xstrcasecmp(slurm_conf.accounting_storage_type,
-			 "accounting_storage/slurmdbd")) {
-		xfree(slurm_conf.accounting_storage_type);
-		slurm_conf.accounting_storage_type =
-			xstrdup("accounting_storage/ctld_relay");
-		acct_storage_g_init();
-	} else {
-			acct_storage_g_init();
-	}
+	acct_storage_g_init();
 
 	slurm_thread_create(&time_limit_thread_id, _step_time_limit_thread,
 			    NULL);
 }
 
-int
-main (int argc, char **argv)
+static void _on_sigalrm(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	debug("Caught SIGALRM. Ignoring.");
+}
+
+static void _on_sigint(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGINT. Shutting down.");
+	conmgr_request_shutdown();
+}
+
+static void _on_sigterm(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGTERM. Shutting down.");
+	conmgr_request_shutdown();
+}
+
+static void _on_sigquit(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGQUIT. Shutting down.");
+	conmgr_request_shutdown();
+}
+
+static void _on_sigtstp(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGTSTP. Ignoring");
+}
+
+static void _on_sighup(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGHUP. Ignoring");
+}
+
+static void _on_sigusr1(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGUSR1. Ignoring.");
+}
+
+static void _on_sigusr2(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGUSR2. Ignoring.");
+}
+
+static void _on_sigpipe(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	info("Caught SIGPIPE. Ignoring.");
+}
+
+static void _on_sigttin(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	debug("Caught SIGTTIN. Ignoring.");
+}
+
+static void _main_thread_init()
+{
+	sigset_t mask;
+
+	/*
+	 * Block SIGCHLD so that we can create a SIGCHLD signalfd later. This
+	 * needs to be done before creating any threads so that all threads
+	 * inherit this signal mask. This ensures that no threads consume
+	 * SIGCHLD, and that a SIGCHLD signalfd can reliably catch SIGCHLD.
+	 */
+	sigemptyset(&mask);
+	sigaddset(&mask, SIGCHLD);
+	if (pthread_sigmask(SIG_BLOCK, &mask, NULL) == -1) {
+		error("pthread_sigmask() failed: %m");
+	}
+}
+
+/*
+ * Validate step record before initialization
+ */
+static int _validate_step(stepd_step_rec_t *step)
+{
+	/*
+	 * --wait-for-children is only supported by the cgroup proctrack plugin.
+	 */
+	if ((step->flags & LAUNCH_WAIT_FOR_CHILDREN)) {
+		char *cgroup_version = autodetect_cgroup_version();
+		if (!xstrstr(slurm_conf.proctrack_type, "proctrack/cgroup")) {
+			error("Failed to validate step %ps. --wait-for-children requires proctrack/cgroup plugin.",
+			      &step->step_id);
+			return SLURM_ERROR;
+		}
+		if (!xstrcmp(cgroup_version, "cgroup/v1")) {
+			error("Failed to validate step %ps. --wait-for-children is not supported in cgroup/v1. cgroup/v2 is required.",
+			      &step->step_id);
+			return SLURM_ERROR;
+		}
+	}
+
+	return SLURM_SUCCESS;
+}
+
+extern int main(int argc, char **argv)
 {
 	log_options_t lopts = LOG_OPTS_INITIALIZER;
 	slurm_addr_t *cli;
 	slurm_msg_t *msg;
 	stepd_step_rec_t *step;
-	int rc = 0;
+	int rc = SLURM_SUCCESS;
+	bool only_mem = true;
+
+	_main_thread_init();
 
 	_process_cmdline(argc, argv);
 
-	run_command_init();
-
-	xsignal_block(slurmstepd_blocked_signals);
 	conf = xmalloc(sizeof(*conf));
 	conf->argv = argv;
 	conf->argc = argc;
@@ -320,9 +400,32 @@ main (int argc, char **argv)
 	/* Receive job parameters from the slurmd */
 	_init_from_slurmd(STDIN_FILENO, argv, &cli, &msg);
 
-	/* Create the stepd_step_rec_t, mostly from info in a
-	 * launch_tasks_request_msg_t or a batch_job_launch_msg_t */
-	if (!(step = _step_setup(cli, msg))) {
+	conmgr_init(0, 0, (conmgr_callbacks_t) {0});
+
+	conmgr_add_work_signal(SIGALRM, _on_sigalrm, NULL);
+	conmgr_add_work_signal(SIGINT, _on_sigint, NULL);
+	conmgr_add_work_signal(SIGTERM, _on_sigterm, NULL);
+	conmgr_add_work_signal(SIGQUIT, _on_sigquit, NULL);
+	conmgr_add_work_signal(SIGTSTP, _on_sigtstp, NULL);
+	conmgr_add_work_signal(SIGHUP, _on_sighup, NULL);
+	conmgr_add_work_signal(SIGUSR1, _on_sigusr1, NULL);
+	conmgr_add_work_signal(SIGUSR2, _on_sigusr2, NULL);
+	conmgr_add_work_signal(SIGPIPE, _on_sigpipe, NULL);
+	conmgr_add_work_signal(SIGTTIN, _on_sigttin, NULL);
+
+	conmgr_run(false);
+
+	if ((run_command_init(argc, argv, conf->stepd_loc) != SLURM_SUCCESS) &&
+	    conf->stepd_loc && conf->stepd_loc[0])
+		fatal("%s: Unable to reliably execute %s",
+		      __func__, conf->stepd_loc);
+
+	/*
+	 * Create the stepd_step_rec_t, mostly from info in a
+	 * launch_tasks_request_msg_t or a batch_job_launch_msg_t, and validate
+	 * the new stepd_step_rec_t before continuing
+	 */
+	if (!(step = _step_setup(cli, msg)) || _validate_step(step)) {
 		rc = SLURM_ERROR;
 		_send_fail_to_slurmd(STDOUT_FILENO, rc);
 		goto ending;
@@ -364,14 +467,19 @@ main (int argc, char **argv)
 	 * and blocks until the step is complete */
 	rc = job_manager(step);
 
-	return stepd_cleanup(msg, step, cli, rc, false);
+	only_mem = false;
 ending:
-	return stepd_cleanup(msg, step, cli, rc, true);
+	stepd_cleanup(msg, step, cli, rc, only_mem);
+
+	conmgr_fini();
+	return rc;
 }
 
-extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
-			 slurm_addr_t *cli, int rc, bool only_mem)
+extern void stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
+			  slurm_addr_t *cli, int rc, bool only_mem)
 {
+	static bool cleanup = false;
+
 	time_limit_thread_shutdown = true;
 
 	slurm_mutex_lock(&cleanup_mutex);
@@ -384,10 +492,18 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 		goto done;
 	}
 
-	if (!only_mem) {
-		if (step->batch)
-			batch_finish(step, rc); /* sends batch complete message */
+	if (!only_mem && step->batch)
+		batch_finish(step, rc); /* sends batch complete message */
 
+	/*
+	 * Call auth_setuid_lock after sending the batch_finish message as in
+	 * there the auth_g_create function is called, and that function uses
+	 * the lock. The lock is needed to ensure that the privileges are not
+	 * dropped from a different thread, like X11 shutdown thread.
+	 */
+	auth_setuid_lock();
+
+	if (!only_mem) {
 		/* signal the message thread to shutdown, and wait for it */
 		if (step->msg_handle)
 			eio_signal_shutdown(step->msg_handle);
@@ -398,24 +514,22 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 
 	/*
 	 * This call is only done once per step since stepd_cleanup is protected
-	 * agains multiple and concurrent calls.
+	 * against multiple and concurrent calls.
 	 */
 	proctrack_g_destroy(step->cont_id);
 
-	if (conf->hwloc_xml)
-		(void)remove(conf->hwloc_xml);
-
 	if (step->container)
 		cleanup_container(step);
 
-	run_command_shutdown();
-
 	if (step->step_id.step_id == SLURM_EXTERN_CONT) {
 		if (container_g_stepd_delete(step->step_id.job_id))
 			error("container_g_stepd_delete(%u): %m",
 			      step->step_id.job_id);
 	}
 
+	auth_setuid_unlock();
+	run_command_shutdown();
+
 	/*
 	 * join() must be done before _step_cleanup() where job_step_ptr is
 	 * freed.
@@ -429,6 +543,7 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 	if (job_step_ptr) {
 		xfree(job_step_ptr->resv_ports);
 		reserve_port_stepmgr_init(job_step_ptr);
+		node_features_free_lists();
 	}
 
 	_step_cleanup(step, msg, rc);
@@ -442,7 +557,6 @@ extern int stepd_cleanup(slurm_msg_t *msg, stepd_step_rec_t *step,
 	xfree(conf->block_map_inv);
 	xfree(conf->conffile);
 	xfree(conf->hostname);
-	xfree(conf->hwloc_xml);
 	xfree(conf->logfile);
 	xfree(conf->node_name);
 	xfree(conf->node_topo_addr);
@@ -464,7 +578,8 @@ done:
 	} else {
 		info("done with step");
 	}
-	return rc;
+
+	conmgr_request_shutdown();
 }
 
 extern void close_slurmd_conn(int rc)
@@ -496,7 +611,7 @@ static slurmd_conf_t *_read_slurmd_conf_lite(int fd)
 	buf_t *buffer = NULL;
 	slurmd_conf_t *confl, *local_conf = NULL;
 	int tmp_int = 0;
-	List tmp_list = NULL;
+	list_t *tmp_list = NULL;
 	assoc_mgr_lock_t locks = { .tres = WRITE_LOCK };
 
 	/*  First check to see if we've already initialized the
@@ -649,10 +764,10 @@ static int _handle_spank_mode(int argc, char **argv)
 	      mode, jobid, uid, gid);
 
 	if (!xstrcmp(mode, "prolog")) {
-		if (spank_job_prolog(jobid, uid, gid) < 0)
+		if (spank_job_prolog(jobid, uid, gid))
 			return -1;
 	} else if (!xstrcmp(mode, "epilog")) {
-		if (spank_job_epilog(jobid, uid, gid) < 0)
+		if (spank_job_epilog(jobid, uid, gid))
 			return -1;
 	} else {
 		error("Invalid mode %s specified!", mode);
@@ -683,14 +798,20 @@ static void _process_cmdline(int argc, char **argv)
 			exit(1);
 		exit(0);
 	}
+	if (run_command_is_launcher(argc, argv)) {
+		run_command_launcher(argc, argv);
+		_exit(127); /* Should not get here */
+	}
 }
 
 static void
 _send_ok_to_slurmd(int sock)
 {
-	/* If running under valgrind/memcheck, this pipe doesn't work correctly
-	 * so just skip it. */
-#if (SLURMSTEPD_MEMCHECK == 0)
+	/*
+	 * If running under memcheck, this pipe doesn't work correctly so just
+	 * skip it.
+	 */
+#if (SLURMSTEPD_MEMCHECK != 1)
 	int ok = SLURM_SUCCESS;
 	safe_write(sock, &ok, sizeof(int));
 	return;
@@ -701,9 +822,11 @@ rwfail:
 
 static void _send_fail_to_slurmd(int sock, int rc)
 {
-	/* If running under valgrind/memcheck, this pipe doesn't work correctly
-	 * so just skip it. */
-#if (SLURMSTEPD_MEMCHECK == 0)
+	/*
+	 * If running under memcheck, this pipe doesn't work correctly so just
+	 * skip it.
+	 */
+#if (SLURMSTEPD_MEMCHECK != 1)
 	safe_write(sock, &rc, sizeof(int));
 	return;
 rwfail:
@@ -714,9 +837,11 @@ rwfail:
 static void
 _got_ack_from_slurmd(int sock)
 {
-	/* If running under valgrind/memcheck, this pipe doesn't work correctly
-	 * so just skip it. */
-#if (SLURMSTEPD_MEMCHECK == 0)
+	/*
+	 * If running under memcheck, this pipe doesn't work correctly so just
+	 * skip it.
+	 */
+#if (SLURMSTEPD_MEMCHECK != 1)
 	int ok;
 	safe_read(sock, &ok, sizeof(int));
 	return;
@@ -767,11 +892,6 @@ _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 	if (!(conf = _read_slurmd_conf_lite(sock)))
 		fatal("Failed to read conf from slurmd");
 
-	/*
-	 * Init select plugin after reading slurm.conf and before receiving step
-	 */
-	select_g_init(false);
-
 	slurm_conf.slurmd_port = conf->port;
 	slurm_conf.slurmd_syslog_debug = conf->syslog_debug;
 	/*
@@ -917,16 +1037,20 @@ _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 
 	_set_job_log_prefix(&step_id);
 
+	if (cgroup_read_state(sock) != SLURM_SUCCESS)
+		fatal("Failed to read cgroup state from slurmd");
+
 	/*
 	 * Init all plugins after receiving the slurm.conf from the slurmd.
 	 */
 	if ((auth_g_init() != SLURM_SUCCESS) ||
+	    (conn_g_init() != SLURM_SUCCESS) ||
 	    (cgroup_g_init() != SLURM_SUCCESS) ||
 	    (hash_g_init() != SLURM_SUCCESS) ||
 	    (acct_gather_conf_init() != SLURM_SUCCESS) ||
 	    (prep_g_init(NULL) != SLURM_SUCCESS) ||
 	    (proctrack_g_init() != SLURM_SUCCESS) ||
-	    (slurmd_task_init() != SLURM_SUCCESS) ||
+	    (task_g_init() != SLURM_SUCCESS) ||
 	    (jobacct_gather_init() != SLURM_SUCCESS) ||
 	    (acct_gather_profile_init() != SLURM_SUCCESS) ||
 	    (job_container_init() != SLURM_SUCCESS) ||
@@ -960,16 +1084,6 @@ _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 	    (mpi_conf_recv_stepd(sock) != SLURM_SUCCESS))
 		fatal("Failed to read MPI conf from slurmd");
 
-	if (!conf->hwloc_xml) {
-		conf->hwloc_xml = xstrdup_printf("%s/hwloc_topo_%u.%u",
-						 conf->spooldir,
-						 step_id.job_id,
-						 step_id.step_id);
-		if (step_id.step_het_comp != NO_VAL)
-			xstrfmtcat(conf->hwloc_xml, ".%u",
-				   step_id.step_het_comp);
-		xstrcat(conf->hwloc_xml, ".xml");
-	}
 	/*
 	 * Swap the field to the srun client version, which will eventually
 	 * end up stored as protocol_version in srun_info_t. It's a hack to
@@ -981,6 +1095,14 @@ _init_from_slurmd(int sock, char **argv, slurm_addr_t **_cli,
 	*_cli = cli;
 	*_msg = msg;
 
+	/*
+	 * When using TLS, slurmstepd messages bound to other nodes are relayed
+	 * through slurmd. This caches slurmd spooldir so that slurmstepd can
+	 * get the address for slurmd's local unix socket.
+	 */
+	if (tls_enabled())
+		stepd_proxy_stepd_init(conf->spooldir);
+
 	return 1;
 
 rwfail:
@@ -1060,7 +1182,7 @@ static stepd_step_rec_t *_step_setup(slurm_addr_t *cli, slurm_msg_t *msg)
 	}
 
 	/*
-	 * Add slurmd node topology informations to job env array
+	 * Add slurmd node topology information to job env array
 	 */
 	env_array_overwrite(&step->env,"SLURM_TOPOLOGY_ADDR",
 			    conf->node_topo_addr);
@@ -1091,25 +1213,13 @@ _step_cleanup(stepd_step_rec_t *step, slurm_msg_t *msg, int rc)
 			stepd_step_rec_destroy(step);
 	}
 
-	if (msg) {
-		/*
-		 * The message cannot be freed until the jobstep is complete
-		 * because the job struct has pointers into the msg, such
-		 * as the switch jobinfo pointer.
-		 */
-		switch(msg->msg_type) {
-		case REQUEST_BATCH_JOB_LAUNCH:
-			slurm_free_job_launch_msg(msg->data);
-			break;
-		case REQUEST_LAUNCH_TASKS:
-			slurm_free_launch_tasks_request_msg(msg->data);
-			break;
-		default:
-			fatal("handle_launch_message: Unrecognized launch RPC");
-			break;
-		}
-		xfree(msg);
-	}
+	/*
+	 * The message cannot be freed until the jobstep is complete
+	 * because the job struct has pointers into the msg, such
+	 * as the switch jobinfo pointer.
+	 */
+	slurm_free_msg(msg);
+
 	jobacctinfo_destroy(step_complete.jobacct);
 }
 #endif
